/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

/* Lecture des fichiers de la base MNIST de caractères manuscrits.

   Cf http://yann.lecun.com/exdb/mnist/

   Il s'agit d'images 28x28, en niveaux de gris (dans 0..255).
*/

#include "prelude.h"
#include "mnist.h"

const int width = 28, height = 28;
const int size = width * height;

int input_uint32(FILE *f) { // big-endian
  int n = 0;
  for (int i = 0; i < 4; i++)
    n = (n << 8) | fgetc(f);
  return n;
}

void mnist_print_pgm(char *file, image *img) {
  FILE *f = fopen(file, "w");
  fprintf(f, "P2\n");
  fprintf(f, "%d %d\n", width, height);
  fprintf(f, "255");
  for (int i = 0; i < size; i++) {
    if (i % width == 0) fprintf(f, "\n");
    fprintf(f, "%d ", img->data[i]);
  }
  fprintf(f, "\n");
  fclose(f);
}

image *mnist_load(char *file, int lo, int hi) {
  char *fileimg = calloc(80, sizeof(char)), *filelab = calloc(80, sizeof(char));
  sprintf(fileimg, "%s-images-idx3-ubyte", file);
  sprintf(filelab, "%s-labels-idx1-ubyte", file);
  FILE *fi = fopen(fileimg, "r"), *fl = fopen(filelab, "r");
  assert(input_uint32(fi) == 0x00000803);
  assert(input_uint32(fl) == 0x00000801);
  int ni = input_uint32(fi);
  assert(0 <= lo && lo <= hi && hi <= ni);
  assert (input_uint32(fi) == height);
  assert (input_uint32(fi) == width);
  assert (input_uint32(fl) == ni);
  fseek(fi, lo * size, SEEK_CUR);
  fseek(fl, lo, SEEK_CUR);
  image *img = calloc(hi - lo, sizeof(struct Image));
  for (int idx = lo; idx < hi; idx++) {
    int i = idx - lo;
    img[i].index = idx;
    img[i].data = malloc(size * sizeof(unsigned char));
    assert(fread(img[i].data, 1, size * sizeof(unsigned char), fi) == size);
    img[i].label = fgetc(fl);
  }
  fclose(fi); fclose(fl);
  return img;
}

void mnist_delete_image(image *img) {
  free(img->data);
  free(img);
}



